`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 11/09/2023 12:37:07 PM
// Design Name: 
// Module Name: top
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module top(
    input clock,            // system clock
    input reset,            // BTNR
    input version,          // BTNU
    input latch,            // BTNL
    input adc_n, adc_p,     // VCAUX 6, P and N
    output [15:0] led,      // 16  onboard LEDs above the switches
    output [6:0] segment,   // 7-segment digit LED display
    output dp,              // "." next to each LED digit
    output [3:0] digit,     // which of the 4 digits to drive
    output [7:0] JB,        // debugging
    output [7:0] JA         //  
    );
    parameter VERSION = 'h0001;
    //
    //  the XADC input clock and an internal clock divider determine the conversion
    //  rate.  we will shoot for 1MSps.  XADC takes 26 clock ticks for a conversion, so
    //  to get the input clock with a divider of 4 we multiply 4x26x1MHz = 104MHz.  
    //
    //  to get such a frequency, use the Xilinx clock wizard.  let's make a 104 clock
    //  for the XADC
    //
    wire locked, clock104;
    clock_104 MYCLOCKS(
        .reset(reset),
        .clk_in1(clock),
        .locked(locked),
        .clk_out1(clock104)
    );
    //
    //  debounce the latch button
    //
    wire latch_level, latch_pulse;
    debouncer DEBOUNCE_LATCH(
        .clock(clock), .button(latch), .level(latch_level), .pulse(latch_pulse) );
    //
    //  next drive the 4 7-segment displays
    //
    wire [15:0] display_this;
    display4 DISPLAY (
        .clk100(clock),
        .number(display_this),
        .digit(digit),
        .segments(segment),
        .period(dp)
        );
    //
    //  here is the XADC block
    //
    wire [6:0] daddr_in = 7'h16;
    wire adc_ready, isbusy, adc_data_ready, eos_out, alarm;
    wire [15:0] adc_data;
    wire [4:0] channel_out;
    myxadc XADC_INST (
        .daddr_in(7'h16),   // specifies vcaux6 pints to digitize
        .dclk_in(clock104),    // 50MHz clock
        .den_in(adc_ready), // tied to adc_ready, tells adc to convert, tieing causes continuous conversions
        .di_in(16'h0),      // to set the data to something, not used here
        .dwe_in(1'b0),      //  set to enable writing to di_in, which we don't want to do
        .vauxp6(adc_p),     //  positive input to digitize
        .vauxn6(adc_n),     //  negative input to digitize
        .busy_out(isbusy),  // tells you the adc is busy converting
        .channel_out(channel_out[4:0]), // for using more than 1 channel, tells you which one.  not used here
        .do_out(adc_data),      // adc value from conversion
        .drdy_out(adc_data_ready),  //tells you valid data is ready to be latched
        .eoc_out(adc_ready),   //  specifies that the ADC is ready (conversion complete)
        .eos_out(eos_out),     //  specifies that conversion sequence is complete
        .alarm_out(alarm),      // OR's output of all internal alarms, not used here
        .vp_in(1'b0),           // dedicated analog input pair for differential, tied to 0 if not used
        .vn_in(1'b0)
    );
    //
    // wait for XADC to tell you something is ready to latch
    //
    reg [15:0] r_adc_data;
    always @ (negedge isbusy) 
        if (reset) r_adc_data <= 16'h0;
        else r_adc_data <= adc_data;

    //
    //  make a ~1Hz clock so we can run the LED display slower
    //
    reg [26:0] counter;
    reg [15:0] s_adc_data;
    always @ (posedge clock) begin
        if (reset) counter <= 0;
        else counter <= counter + 1;
        end
    wire clock_1hz = counter[26];
    always @ (posedge clock_1hz) s_adc_data <= r_adc_data;
    assign display_this = version ? VERSION : s_adc_data;
    //
    //  latching the adc value for LED output from latch pushbutton 
    //
    reg [15:0] latched_adc;          // latched adc data
    always @ (posedge latch_pulse)
        if (reset) latched_adc <= 0;
        else latched_adc <= r_adc_data;
    assign led = latched_adc;
    //
    //  some output for debugging if necessary
    //
    assign JA = {3'b000,eos_out,adc_data_ready,adc_ready,isbusy,clock104};
    assign JB = {r_adc_data[15:12],adc_data[15:12]};
endmodule

